@@ -13,12 +13,11 @@ use Carp 'croak';
 use Mojo::Collection;
 use Mojo::DOM::CSS;
 use Mojo::DOM::HTML;
-use Mojo::Util 'squish';
-use Scalar::Util qw(blessed weaken);
+use Scalar::Util 'weaken';
 
-sub all_text { shift->_all_text(1, @_) }
+sub all_text { _text(_nodes(shift->tree), 1) }
 
-sub ancestors { _select($_[0]->_collect($_[0]->_ancestors), $_[1]) }
+sub ancestors { _select($_[0]->_collect([$_[0]->_ancestors]), $_[1]) }
 
 sub append { shift->_add(1, @_) }
 sub append_content { shift->_content(1, 0, @_) }
@@ -58,7 +57,7 @@ sub content {
   if ($type eq 'root' || $type eq 'tag') {
     return $self->_content(0, 1, @_) if @_;
     my $html = Mojo::DOM::HTML->new(xml => $self->xml);
-    return join '', map { $html->tree($_)->render } _nodes($self->tree);
+    return join '', map { $html->tree($_)->render } @{_nodes($self->tree)};
   }
 
   return $self->tree->[1] unless @_;
@@ -68,10 +67,10 @@ sub content {
 
 sub descendant_nodes { $_[0]->_collect(_all(_nodes($_[0]->tree))) }
 
-sub find { $_[0]->_collect(@{$_[0]->_css->select($_[1])}) }
+sub find { $_[0]->_collect($_[0]->_css->select($_[1])) }
 
-sub following { _select($_[0]->_collect(@{$_[0]->_siblings(1)->[1]}), $_[1]) }
-sub following_nodes { $_[0]->_collect(@{$_[0]->_siblings->[1]}) }
+sub following { _select($_[0]->_collect($_[0]->_siblings(1, 1)), $_[1]) }
+sub following_nodes { $_[0]->_collect($_[0]->_siblings(0, 1)) }
 
 sub matches { shift->_css->matches(@_) }
 
@@ -101,32 +100,32 @@ sub new {
   return @_ ? $self->parse(@_) : $self;
 }
 
-sub next      { $_[0]->_maybe($_[0]->_siblings(1, 0)->[1]) }
-sub next_node { $_[0]->_maybe($_[0]->_siblings(0, 0)->[1]) }
+sub next      { $_[0]->_maybe($_[0]->_siblings(1, 1, 0)) }
+sub next_node { $_[0]->_maybe($_[0]->_siblings(0, 1, 0)) }
 
 sub parent {
   my $self = shift;
-  return undef if $self->tree->[0] eq 'root';
-  return $self->_build($self->_parent, $self->xml);
+  return undef if (my $tree = $self->tree)->[0] eq 'root';
+  return $self->_build(_parent($tree), $self->xml);
 }
 
 sub parse { shift->_delegate(parse => @_) }
 
-sub preceding { _select($_[0]->_collect(@{$_[0]->_siblings(1)->[0]}), $_[1]) }
-sub preceding_nodes { $_[0]->_collect(@{$_[0]->_siblings->[0]}) }
+sub preceding { _select($_[0]->_collect($_[0]->_siblings(1, 0)), $_[1]) }
+sub preceding_nodes { $_[0]->_collect($_[0]->_siblings(0)) }
 
 sub prepend { shift->_add(0, @_) }
 sub prepend_content { shift->_content(0, 0, @_) }
 
-sub previous      { $_[0]->_maybe($_[0]->_siblings(1, -1)->[0]) }
-sub previous_node { $_[0]->_maybe($_[0]->_siblings(0, -1)->[0]) }
+sub previous      { $_[0]->_maybe($_[0]->_siblings(1, 0, -1)) }
+sub previous_node { $_[0]->_maybe($_[0]->_siblings(0, 0, -1)) }
 
 sub remove { shift->replace('') }
 
 sub replace {
   my ($self, $new) = @_;
   return $self->parse($new) if (my $tree = $self->tree)->[0] eq 'root';
-  return $self->_replace($self->_parent, $tree, $self->_parse($new));
+  return $self->_replace(_parent($tree), $tree, _nodes($self->_parse($new)));
 }
 
 sub root {
@@ -138,7 +137,7 @@ sub root {
 sub strip {
   my $self = shift;
   return $self if (my $tree = $self->tree)->[0] ne 'tag';
-  return $self->_replace($tree->[3], $tree, ['root', _nodes($tree)]);
+  return $self->_replace($tree->[3], $tree, _nodes($tree));
 }
 
 sub tag {
@@ -151,7 +150,7 @@ sub tag {
 
 sub tap { shift->Mojo::Base::tap(@_) }
 
-sub text { shift->_all_text(0, @_) }
+sub text { _text(_nodes(shift->tree), 0) }
 
 sub to_string { shift->_delegate('render') }
 
@@ -159,6 +158,28 @@ sub tree { shift->_delegate(tree => @_) }
 
 sub type { shift->tree->[0] }
 
+sub val {
+  my $self = shift;
+
+  # "option"
+  return $self->{value} // $self->text if (my $tag = $self->tag) eq 'option';
+
+  # "input" ("type=checkbox" and "type=radio")
+  my $type = $self->{type} // '';
+  return $self->{value} // 'on'
+    if $tag eq 'input' && ($type eq 'radio' || $type eq 'checkbox');
+
+  # "textarea", "input" or "button"
+  return $tag eq 'textarea' ? $self->text : $self->{value} if $tag ne 'select';
+
+  # "select"
+  my $v = $self->find('option:checked:not([disabled])')
+    ->grep(sub { !$_->ancestors('optgroup[disabled]')->size })->map('val');
+  return exists $self->{multiple} ? $v->size ? $v->to_array : undef : $v->last;
+}
+
+sub with_roles { shift->Mojo::Base::with_roles(@_) }
+
 sub wrap         { shift->_wrap(0, @_) }
 sub wrap_content { shift->_wrap(1, @_) }
 
@@ -169,33 +190,23 @@ sub _add {
 
   return $self if (my $tree = $self->tree)->[0] eq 'root';
 
-  my $parent = $self->_parent;
+  my $parent = _parent($tree);
   splice @$parent, _offset($parent, $tree) + $offset, 0,
-    _link($self->_parse($new), $parent);
+    @{_link($parent, _nodes($self->_parse($new)))};
 
   return $self;
 }
 
 sub _all {
-  map { $_->[0] eq 'tag' ? ($_, _all(_nodes($_))) : ($_) } @_;
-}
-
-sub _all_text {
-  my ($self, $recurse, $trim) = @_;
-
-  # Detect "pre" tag
-  my $tree = $self->tree;
-  $trim = 1 unless defined $trim;
-  map { $_->[1] eq 'pre' and $trim = 0 } $self->_ancestors, $tree
-    if $trim && $tree->[0] ne 'root';
-
-  return _text([_nodes($tree)], $recurse, $trim);
+  my $nodes = shift;
+  @$nodes = map { $_->[0] eq 'tag' ? ($_, @{_all(_nodes($_))}) : ($_) } @$nodes;
+  return $nodes;
 }
 
 sub _ancestors {
   my ($self, $root) = @_;
 
-  return unless my $tree = $self->_parent;
+  return () unless my $tree = _parent($self->tree);
   my @ancestors;
   do { push @ancestors, $tree }
     while ($tree->[0] eq 'tag') && ($tree = $tree->[3]);
@@ -205,9 +216,9 @@ sub _ancestors {
 sub _build { shift->new->tree(shift)->xml(shift) }
 
 sub _collect {
-  my $self = shift;
-  my $xml  = $self->xml;
-  return Mojo::Collection->new(map { $self->_build($_, $xml) } @_);
+  my ($self, $nodes) = (shift, shift // []);
+  my $xml = $self->xml;
+  return Mojo::Collection->new(map { $self->_build($_, $xml) } @$nodes);
 }
 
 sub _content {
@@ -216,12 +227,12 @@ sub _content {
   my $tree = $self->tree;
   unless ($tree->[0] eq 'root' || $tree->[0] eq 'tag') {
     my $old = $self->content;
-    return $self->content($start ? "$old$new" : "$new$old");
+    return $self->content($start ? $old . $new : $new . $old);
   }
 
   $start  = $start  ? ($#$tree + 1) : _start($tree);
   $offset = $offset ? $#$tree       : 0;
-  splice @$tree, $start, $offset, _link($self->_parse($new), $tree);
+  splice @$tree, $start, $offset, @{_link($tree, _nodes($self->_parse($new)))};
 
   return $self;
 }
@@ -236,25 +247,24 @@ sub _delegate {
 }
 
 sub _link {
-  my ($children, $parent) = @_;
+  my ($parent, $children) = @_;
 
   # Link parent to children
-  my @new = @$children[1 .. $#$children];
-  for my $node (@new) {
+  for my $node (@$children) {
     my $offset = $node->[0] eq 'tag' ? 3 : 2;
     $node->[$offset] = $parent;
     weaken $node->[$offset];
   }
 
-  return @new;
+  return $children;
 }
 
 sub _maybe { $_[1] ? $_[0]->_build($_[1], $_[0]->xml) : undef }
 
 sub _nodes {
-  return unless my $tree = shift;
+  return () unless my $tree = shift;
   my @nodes = @$tree[_start($tree) .. $#$tree];
-  return shift() ? grep { $_->[0] eq 'tag' } @nodes : @nodes;
+  return shift() ? [grep { $_->[0] eq 'tag' } @nodes] : \@nodes;
 }
 
 sub _offset {
@@ -264,72 +274,51 @@ sub _offset {
   return $i;
 }
 
-sub _parent { $_[0]->tree->[$_[0]->type eq 'tag' ? 3 : 2] }
+sub _parent { $_[0]->[$_[0][0] eq 'tag' ? 3 : 2] }
 
 sub _parse { Mojo::DOM::HTML->new(xml => shift->xml)->parse(shift)->tree }
 
 sub _replace {
-  my ($self, $parent, $tree, $new) = @_;
-  splice @$parent, _offset($parent, $tree), 1, _link($new, $parent);
+  my ($self, $parent, $child, $nodes) = @_;
+  splice @$parent, _offset($parent, $child), 1, @{_link($parent, $nodes)};
   return $self->parent;
 }
 
-sub _select {
-  my ($collection, $selector) = @_;
-  return $collection unless $selector;
-  return $collection->new(grep { $_->matches($selector) } @$collection);
-}
+sub _select { $_[1] ? $_[0]->grep(matches => $_[1]) : $_[0] }
 
 sub _siblings {
-  my ($self, $tags, $i) = @_;
+  my ($self, $tags, $tail, $i) = @_;
 
-  return [] unless my $parent = $self->parent;
+  return defined $i ? undef : [] if (my $tree = $self->tree)->[0] eq 'root';
 
-  my $tree = $self->tree;
-  my (@before, @after, $match);
-  for my $node (_nodes($parent->tree)) {
-    ++$match and next if !$match && $node eq $tree;
-    next if $tags && $node->[0] ne 'tag';
-    $match ? push @after, $node : push @before, $node;
-  }
+  my $nodes = _nodes(_parent($tree));
+  my $match = -1;
+  defined($match++) and $_ eq $tree and last for @$nodes;
 
-  return defined $i ? [$before[$i], $after[$i]] : [\@before, \@after];
+  if ($tail) { splice @$nodes, 0, $match + 1 }
+  else       { splice @$nodes, $match, ($#$nodes + 1) - $match }
+
+  @$nodes = grep { $_->[0] eq 'tag' } @$nodes if $tags;
+
+  return defined $i ? $i == -1 && !@$nodes ? undef : $nodes->[$i] : $nodes;
 }
 
 sub _start { $_[0][0] eq 'root' ? 1 : 4 }
 
 sub _text {
-  my ($nodes, $recurse, $trim) = @_;
-
-  # Merge successive text nodes
-  my $i = 0;
-  while (my $next = $nodes->[$i + 1]) {
-    ++$i and next unless $nodes->[$i][0] eq 'text' && $next->[0] eq 'text';
-    splice @$nodes, $i, 2, ['text', $nodes->[$i][1] . $next->[1]];
-  }
+  my ($nodes, $all) = @_;
 
   my $text = '';
-  for my $node (@$nodes) {
+  while (my $node = shift @$nodes) {
     my $type = $node->[0];
 
     # Text
-    my $chunk = '';
-    if ($type eq 'text') { $chunk = $trim ? squish $node->[1] : $node->[1] }
-
-    # CDATA or raw text
-    elsif ($type eq 'cdata' || $type eq 'raw') { $chunk = $node->[1] }
-
-    # Nested tag
-    elsif ($type eq 'tag' && $recurse) {
-      no warnings 'recursion';
-      $chunk = _text([_nodes($node)], 1, $node->[1] eq 'pre' ? 0 : $trim);
+    if ($type eq 'text' || $type eq 'cdata' || $type eq 'raw') {
+      $text .= $node->[1];
     }
 
-    # Add leading whitespace if punctuation allows it
-    $chunk = " $chunk" if $text =~ /\S\z/ && $chunk =~ /^[^.!?,;:\s]+/;
-
-    # Trim whitespace blocks
-    $text .= $chunk if $chunk =~ /\S+/ || !$trim;
+    # Nested tag
+    elsif ($type eq 'tag' && $all) { unshift @$nodes, @{_nodes($node)} }
   }
 
   return $text;
@@ -338,25 +327,25 @@ sub _text {
 sub _wrap {
   my ($self, $content, $new) = @_;
 
-  $content = 1 if (my $tree = $self->tree)->[0] eq 'root';
-  $content = 0 if $tree->[0] ne 'root' && $tree->[0] ne 'tag';
+  return $self if (my $tree = $self->tree)->[0] eq 'root' && !$content;
+  return $self if $tree->[0] ne 'root' && $tree->[0] ne 'tag' && $content;
 
   # Find innermost tag
   my $current;
   my $first = $new = $self->_parse($new);
-  $current = $first while $first = (_nodes($first, 1))[0];
+  $current = $first while $first = _nodes($first, 1)->[0];
   return $self unless $current;
 
   # Wrap content
   if ($content) {
-    push @$current, _link(['root', _nodes($tree)], $current);
-    splice @$tree, _start($tree), $#$tree, _link($new, $tree);
+    push @$current, @{_link($current, _nodes($tree))};
+    splice @$tree, _start($tree), $#$tree, @{_link($tree, _nodes($new))};
     return $self;
   }
 
   # Wrap element
-  $self->_replace($self->_parent, $tree, $new);
-  push @$current, _link(['root', $tree], $current);
+  $self->_replace(_parent($tree), $tree, _nodes($new));
+  push @$current, @{_link($current, [$tree])};
   return $self;
 }
 
@@ -401,27 +390,57 @@ L<Mojo::DOM> is a minimalistic and relaxed HTML/XML DOM parser with CSS
 selector support. It will even try to interpret broken HTML and XML, so you
 should not use it for validation.
 
+=head1 NODES AND ELEMENTS
+
+When we parse an HTML/XML fragment, it gets turned into a tree of nodes.
+
+  <!DOCTYPE html>
+  <html>
+    <head><title>Hello</title></head>
+    <body>World!</body>
+  </html>
+
+There are currently eight different kinds of nodes, C<cdata>, C<comment>,
+C<doctype>, C<pi>, C<raw>, C<root>, C<tag> and C<text>. Elements are nodes of
+the type C<tag>.
+
+  root
+  |- doctype (html)
+  +- tag (html)
+     |- tag (head)
+     |  +- tag (title)
+     |     +- raw (Hello)
+     +- tag (body)
+        +- text (World!)
+
+While all node types are represented as L<Mojo::DOM> objects, some methods like
+L</"attr"> and L</"namespace"> only apply to elements.
+
 =head1 CASE-SENSITIVITY
 
 L<Mojo::DOM> defaults to HTML semantics, that means all tags and attribute
 names are lowercased and selectors need to be lowercase as well.
 
+  # HTML semantics
   my $dom = Mojo::DOM->new('<P ID="greeting">Hi!</P>');
   say $dom->at('p[id]')->text;
 
-If XML processing instructions are found, the parser will automatically switch
-into XML mode and everything becomes case-sensitive.
+If an XML declaration is found, the parser will automatically switch into XML
+mode and everything becomes case-sensitive.
 
+  # XML semantics
   my $dom = Mojo::DOM->new('<?xml version="1.0"?><P ID="greeting">Hi!</P>');
   say $dom->at('P[ID]')->text;
 
-XML detection can also be disabled with the L</"xml"> method.
-
-  # Force XML semantics
-  $dom->xml(1);
+HTML or XML semantics can also be forced with the L</"xml"> method.
 
   # Force HTML semantics
-  $dom->xml(0);
+  my $dom = Mojo::DOM->new->xml(0)->parse('<P ID="greeting">Hi!</P>');
+  say $dom->at('p[id]')->text;
+
+  # Force XML semantics
+  my $dom = Mojo::DOM->new->xml(1)->parse('<P ID="greeting">Hi!</P>');
+  say $dom->at('P[ID]')->text;
 
 =head1 METHODS
 
@@ -429,17 +448,12 @@ L<Mojo::DOM> implements the following methods.
 
 =head2 all_text
 
-  my $trimmed   = $dom->all_text;
-  my $untrimmed = $dom->all_text(0);
+  my $text = $dom->all_text;
 
-Extract text content from all descendant nodes of this element, smart
-whitespace trimming is enabled by default.
-
-  # "foo bar baz"
-  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->all_text;
+Extract text content from all descendant nodes of this element.
 
   # "foo\nbarbaz\n"
-  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->all_text(0);
+  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->all_text;
 
 =head2 ancestors
 
@@ -457,7 +471,7 @@ All selectors from L<Mojo::DOM::CSS/"SELECTORS"> are supported.
 
   $dom = $dom->append('<p>I ♥ Mojolicious!</p>');
 
-Append HTML/XML fragment to this node.
+Append HTML/XML fragment to this node (for all node types other than C<root>).
 
   # "<div><h1>Test</h1><h2>123</h2></div>"
   $dom->parse('<div><h1>Test</h1></div>')
@@ -490,8 +504,8 @@ node's content.
   my $result = $dom->at('div ~ p');
 
 Find first descendant element of this element matching the CSS selector and
-return it as a L<Mojo::DOM> object or return C<undef> if none could be found.
-All selectors from L<Mojo::DOM::CSS/"SELECTORS"> are supported.
+return it as a L<Mojo::DOM> object, or C<undef> if none could be found. All
+selectors from L<Mojo::DOM::CSS/"SELECTORS"> are supported.
 
   # Find first element with "svg" namespace definition
   my $namespace = $dom->at('[xmlns\:svg]')->{'xmlns:svg'};
@@ -505,6 +519,12 @@ All selectors from L<Mojo::DOM::CSS/"SELECTORS"> are supported.
 
 This element's attributes.
 
+  # Remove an attribute
+  delete $dom->attr->{id};
+
+  # Attribute without value
+  $dom->attr(selected => undef);
+
   # List id attributes
   say $dom->find('*')->map(attr => 'id')->compact->join("\n");
 
@@ -518,8 +538,11 @@ as L<Mojo::DOM> objects.
   # "<p><b>123</b></p>"
   $dom->parse('<p>Test<b>123</b></p>')->at('p')->child_nodes->first->remove;
 
-  # "<!-- Test -->"
-  $dom->parse('<!-- Test --><b>123</b>')->child_nodes->first;
+  # "<!DOCTYPE html>"
+  $dom->parse('<!DOCTYPE html><b>123</b>')->child_nodes->first;
+
+  # " Test "
+  $dom->parse('<b>123</b><!-- Test -->')->child_nodes->last->content;
 
 =head2 children
 
@@ -638,7 +661,7 @@ L<Mojo::DOM::CSS/"SELECTORS"> are supported.
 
   my $namespace = $dom->namespace;
 
-Find this element's namespace or return C<undef> if none could be found.
+Find this element's namespace, or return C<undef> if none could be found.
 
   # Find namespace for an element with namespace prefix
   my $namespace = $dom->at('svg > svg\:circle')->namespace;
@@ -658,7 +681,7 @@ fragment if necessary.
 
   my $sibling = $dom->next;
 
-Return L<Mojo::DOM> object for next sibling element or C<undef> if there are no
+Return L<Mojo::DOM> object for next sibling element, or C<undef> if there are no
 more siblings.
 
   # "<h2>123</h2>"
@@ -668,7 +691,7 @@ more siblings.
 
   my $sibling = $dom->next_node;
 
-Return L<Mojo::DOM> object for next sibling node or C<undef> if there are no
+Return L<Mojo::DOM> object for next sibling node, or C<undef> if there are no
 more siblings.
 
   # "456"
@@ -683,9 +706,12 @@ more siblings.
 
   my $parent = $dom->parent;
 
-Return L<Mojo::DOM> object for parent of this node or C<undef> if this node has
+Return L<Mojo::DOM> object for parent of this node, or C<undef> if this node has
 no parent.
 
+  # "<b><i>Test</i></b>"
+  $dom->parse('<p><b><i>Test</i></b></p>')->at('i')->parent;
+
 =head2 parse
 
   $dom = $dom->parse('<foo bar="baz">I ♥ Mojolicious!</foo>');
@@ -693,7 +719,7 @@ no parent.
 Parse HTML/XML fragment with L<Mojo::DOM::HTML>.
 
   # Parse XML
-  my $dom = Mojo::DOM->new->xml(1)->parse($xml);
+  my $dom = Mojo::DOM->new->xml(1)->parse('<foo>I ♥ Mojolicious!</foo>');
 
 =head2 preceding
 
@@ -721,7 +747,7 @@ node as L<Mojo::DOM> objects.
 
   $dom = $dom->prepend('<p>I ♥ Mojolicious!</p>');
 
-Prepend HTML/XML fragment to this node.
+Prepend HTML/XML fragment to this node (for all node types other than C<root>).
 
   # "<div><h1>Test</h1><h2>123</h2></div>"
   $dom->parse('<div><h2>123</h2></div>')
@@ -753,7 +779,7 @@ node's content.
 
   my $sibling = $dom->previous;
 
-Return L<Mojo::DOM> object for previous sibling element or C<undef> if there
+Return L<Mojo::DOM> object for previous sibling element, or C<undef> if there
 are no more siblings.
 
   # "<h1>Test</h1>"
@@ -763,7 +789,7 @@ are no more siblings.
 
   my $sibling = $dom->previous_node;
 
-Return L<Mojo::DOM> object for previous sibling node or C<undef> if there are
+Return L<Mojo::DOM> object for previous sibling node, or C<undef> if there are
 no more siblings.
 
   # "123"
@@ -778,7 +804,7 @@ no more siblings.
 
   my $parent = $dom->remove;
 
-Remove this node and return L</"parent">.
+Remove this node and return L</"root"> (for C<root> nodes) or L</"parent">.
 
   # "<div></div>"
   $dom->parse('<div><h1>Test</h1></div>')->at('h1')->remove;
@@ -791,7 +817,8 @@ Remove this node and return L</"parent">.
 
   my $parent = $dom->replace('<div>I ♥ Mojolicious!</div>');
 
-Replace this node with HTML/XML fragment and return L</"parent">.
+Replace this node with HTML/XML fragment and return L</"root"> (for C<root>
+nodes) or L</"parent">.
 
   # "<div><h2>123</h2></div>"
   $dom->parse('<div><h1>Test</h1></div>')->at('h1')->replace('<h2>123</h2>');
@@ -804,7 +831,7 @@ Replace this node with HTML/XML fragment and return L</"parent">.
 
   my $root = $dom->root;
 
-Return L<Mojo::DOM> object for root node.
+Return L<Mojo::DOM> object for C<root> node.
 
 =head2 strip
 
@@ -833,17 +860,15 @@ Alias for L<Mojo::Base/"tap">.
 
 =head2 text
 
-  my $trimmed   = $dom->text;
-  my $untrimmed = $dom->text(0);
+  my $text = $dom->text;
 
-Extract text content from this element only (not including child elements),
-smart whitespace trimming is enabled by default.
+Extract text content from this element only (not including child elements).
 
-  # "foo baz"
-  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->text;
+  # "bar"
+  $dom->parse("<div>foo<p>bar</p>baz</div>")->at('p')->text;
 
   # "foo\nbaz\n"
-  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->text(0);
+  $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->at('div')->text;
 
 =head2 to_string
 
@@ -869,12 +894,74 @@ carefully since it is very dynamic.
 This node's type, usually C<cdata>, C<comment>, C<doctype>, C<pi>, C<raw>,
 C<root>, C<tag> or C<text>.
 
+  # "cdata"
+  $dom->parse('<![CDATA[Test]]>')->child_nodes->first->type;
+
+  # "comment"
+  $dom->parse('<!-- Test -->')->child_nodes->first->type;
+
+  # "doctype"
+  $dom->parse('<!DOCTYPE html>')->child_nodes->first->type;
+
+  # "pi"
+  $dom->parse('<?xml version="1.0"?>')->child_nodes->first->type;
+
+  # "raw"
+  $dom->parse('<title>Test</title>')->at('title')->child_nodes->first->type;
+
+  # "root"
+  $dom->parse('<p>Test</p>')->type;
+
+  # "tag"
+  $dom->parse('<p>Test</p>')->at('p')->type;
+
+  # "text"
+  $dom->parse('<p>Test</p>')->at('p')->child_nodes->first->type;
+
+=head2 val
+
+  my $value = $dom->val;
+
+Extract value from form element (such as C<button>, C<input>, C<option>,
+C<select> and C<textarea>), or return C<undef> if this element has no value. In
+the case of C<select> with C<multiple> attribute, find C<option> elements with
+C<selected> attribute and return an array reference with all values, or C<undef>
+if none could be found.
+
+  # "a"
+  $dom->parse('<input name=test value=a>')->at('input')->val;
+
+  # "b"
+  $dom->parse('<textarea>b</textarea>')->at('textarea')->val;
+
+  # "c"
+  $dom->parse('<option value="c">Test</option>')->at('option')->val;
+
+  # "d"
+  $dom->parse('<select><option selected>d</option></select>')
+    ->at('select')->val;
+
+  # "e"
+  $dom->parse('<select multiple><option selected>e</option></select>')
+    ->at('select')->val->[0];
+
+  # "on"
+  $dom->parse('<input name=test type=checkbox>')->at('input')->val;
+
+=head2 with_roles
+
+  my $new_class = Mojo::DOM->with_roles('Mojo::DOM::Role::One');
+  my $new_class = Mojo::DOM->with_roles('+One', '+Two');
+  $dom          = $dom->with_roles('+One', '+Two');
+
+Alias for L<Mojo::Base/"with_roles">.
+
 =head2 wrap
 
   $dom = $dom->wrap('<div></div>');
 
-Wrap HTML/XML fragment around this node, placing it as the last child of the
-first innermost element.
+Wrap HTML/XML fragment around this node (for all node types other than C<root>),
+placing it as the last child of the first innermost element.
 
   # "<p>123<b>Test</b></p>"
   $dom->parse('<b>Test</b>')->at('b')->wrap('<p>123</p>')->root;
@@ -892,8 +979,8 @@ first innermost element.
 
   $dom = $dom->wrap_content('<div></div>');
 
-Wrap HTML/XML fragment around this node's content, placing it as the last
-children of the first innermost element.
+Wrap HTML/XML fragment around this node's content (for C<root> and C<tag>
+nodes), placing it as the last children of the first innermost element.
 
   # "<p><b>123Test</b></p>"
   $dom->parse('<p>Test<p>')->at('p')->wrap_content('<b>123</b>')->root;
@@ -907,7 +994,7 @@ children of the first innermost element.
   $dom     = $dom->xml($bool);
 
 Disable HTML semantics in parser and activate case-sensitivity, defaults to
-auto detection based on processing instructions.
+auto-detection based on XML declarations.
 
 =head1 OPERATORS
 
@@ -945,6 +1032,6 @@ Alias for L</"to_string">.
 
 =head1 SEE ALSO
 
-L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
 
 =cut
