@@ -7,11 +7,14 @@ use Digest::MD5 qw(md5 md5_hex);
 use Digest::SHA qw(hmac_sha1_hex sha1 sha1_hex);
 use Encode 'find_encoding';
 use Exporter 'import';
+use Getopt::Long 'GetOptionsFromArray';
 use IO::Poll qw(POLLIN POLLPRI);
 use List::Util 'min';
 use MIME::Base64 qw(decode_base64 encode_base64);
+use Pod::Usage 'pod2usage';
 use Symbol 'delete_package';
-use Time::HiRes ();
+use Time::HiRes        ();
+use Unicode::Normalize ();
 
 # Check for monotonic clock support
 use constant MONOTONIC =>
@@ -28,13 +31,18 @@ use constant {
   PC_INITIAL_N    => 128
 };
 
+# Supported on Perl 5.22+
+my $NAME
+  = eval { require Sub::Util; Sub::Util->can('set_subname') } || sub { $_[1] };
+
 # To generate a new HTML entity table run this command
 # perl examples/entities.pl
 my %ENTITIES;
-while (my $line = <DATA>) {
+for my $line (split "\n", join('', <DATA>)) {
   next unless $line =~ /^(\S+)\s+U\+(\S+)(?:\s+U\+(\S+))?/;
   $ENTITIES{$1} = defined $3 ? (chr(hex $2) . chr(hex $3)) : chr(hex $2);
 }
+close DATA;
 
 # Characters that should be escaped in XML
 my %XML = (
@@ -48,20 +56,35 @@ my %XML = (
 # "Sun, 06 Nov 1994 08:49:37 GMT" and "Sunday, 06-Nov-94 08:49:37 GMT"
 my $EXPIRES_RE = qr/(\w+\W+\d+\W+\w+\W+\d+\W+\d+:\d+:\d+\W*\w+)/;
 
-# Encoding cache
-my %CACHE;
+# HTML entities
+my $ENTITY_RE = qr/&(?:\#((?:[0-9]{1,7}|x[0-9a-fA-F]{1,6}));|(\w+[;=]?))/;
+
+# Encoding and pattern cache
+my (%ENCODING, %PATTERN);
 
 our @EXPORT_OK = (
   qw(b64_decode b64_encode camelize class_to_file class_to_path decamelize),
-  qw(decode deprecated dumper encode hmac_sha1_sum html_unescape md5_bytes),
-  qw(md5_sum monkey_patch punycode_decode punycode_encode quote),
-  qw(secure_compare sha1_bytes sha1_sum slurp split_cookie_header),
-  qw(split_header spurt squish steady_time tablify term_escape trim unindent),
-  qw(unquote url_escape url_unescape xml_escape xor_encode xss_escape)
+  qw(decode deprecated dumper encode extract_usage getopt hmac_sha1_sum),
+  qw(html_attr_unescape html_unescape md5_bytes md5_sum monkey_patch),
+  qw(punycode_decode punycode_encode quote secure_compare sha1_bytes sha1_sum),
+  qw(slugify split_cookie_header split_header steady_time tablify term_escape),
+  qw(trim unindent unquote url_escape url_unescape xml_escape xor_encode)
 );
 
-sub b64_decode { decode_base64 $_[0] }
-sub b64_encode { encode_base64 $_[0], $_[1] }
+# Aliases
+monkey_patch(__PACKAGE__, 'b64_decode',    \&decode_base64);
+monkey_patch(__PACKAGE__, 'b64_encode',    \&encode_base64);
+monkey_patch(__PACKAGE__, 'hmac_sha1_sum', \&hmac_sha1_hex);
+monkey_patch(__PACKAGE__, 'md5_bytes',     \&md5);
+monkey_patch(__PACKAGE__, 'md5_sum',       \&md5_hex);
+monkey_patch(__PACKAGE__, 'sha1_bytes',    \&sha1);
+monkey_patch(__PACKAGE__, 'sha1_sum',      \&sha1_hex);
+
+# Use a monotonic clock if possible
+monkey_patch(__PACKAGE__, 'steady_time',
+  MONOTONIC
+  ? sub () { Time::HiRes::clock_gettime(Time::HiRes::CLOCK_MONOTONIC()) }
+  : \&Time::HiRes::time);
 
 sub camelize {
   my $str = shift;
@@ -108,38 +131,49 @@ sub dumper {
   Data::Dumper->new([@_])->Indent(1)->Sortkeys(1)->Terse(1)->Useqq(1)->Dump;
 }
 
-sub encode { _encoding($_[0])->encode("$_[1]") }
+sub encode { _encoding($_[0])->encode("$_[1]", 0) }
 
-sub hmac_sha1_sum { hmac_sha1_hex @_ }
+sub extract_usage {
+  my $file = @_ ? "$_[0]" : (caller)[1];
 
-sub html_unescape {
-  my $str = shift;
-  $str =~ s/&(?:\#((?:\d{1,7}|x[0-9a-fA-F]{1,6}));|(\w+;?))/_decode($1, $2)/ge;
-  return $str;
+  open my $handle, '>', \my $output;
+  pod2usage -exitval => 'noexit', -input => $file, -output => $handle;
+  $output =~ s/^.*\n|\n$//;
+  $output =~ s/\n$//;
+
+  return unindent($output);
 }
 
-sub md5_bytes { md5 @_ }
-sub md5_sum   { md5_hex @_ }
+sub getopt {
+  my ($array, $opts) = map { ref $_[0] eq 'ARRAY' ? shift : $_ } \@ARGV, [];
+  my $save = Getopt::Long::Configure(qw(default no_auto_abbrev no_ignore_case),
+    @$opts);
+  GetOptionsFromArray $array, @_;
+  Getopt::Long::Configure($save);
+}
 
-# Declared in Mojo::Base to avoid circular require problems
-sub monkey_patch { Mojo::Base::_monkey_patch(@_) }
+sub html_attr_unescape { _html(shift, 1) }
+sub html_unescape      { _html(shift, 0) }
+
+sub monkey_patch {
+  my ($class, %patch) = @_;
+  no strict 'refs';
+  no warnings 'redefine';
+  *{"${class}::$_"} = $NAME->("${class}::$_", $patch{$_}) for keys %patch;
+}
 
 # Direct translation of RFC 3492
 sub punycode_decode {
   my $input = shift;
   use integer;
 
-  my $n    = PC_INITIAL_N;
-  my $i    = 0;
-  my $bias = PC_INITIAL_BIAS;
-  my @output;
+  my ($n, $i, $bias, @output) = (PC_INITIAL_N, 0, PC_INITIAL_BIAS);
 
   # Consume all code points before the last delimiter
   push @output, split('', $1) if $input =~ s/(.*)\x2d//s;
 
   while (length $input) {
-    my $oldi = $i;
-    my $w    = 1;
+    my ($oldi, $w) = ($i, 1);
 
     # Base to infinity in steps of base
     for (my $k = PC_BASE; 1; $k += PC_BASE) {
@@ -166,25 +200,20 @@ sub punycode_encode {
   my $output = shift;
   use integer;
 
-  my $n     = PC_INITIAL_N;
-  my $delta = 0;
-  my $bias  = PC_INITIAL_BIAS;
+  my ($n, $delta, $bias) = (PC_INITIAL_N, 0, PC_INITIAL_BIAS);
 
   # Extract basic code points
-  my $len   = length $output;
   my @input = map {ord} split '', $output;
-  my @chars = sort grep { $_ >= PC_INITIAL_N } @input;
   $output =~ s/[^\x00-\x7f]+//gs;
   my $h = my $basic = length $output;
   $output .= "\x2d" if $basic > 0;
 
-  for my $m (@chars) {
+  for my $m (sort grep { $_ >= PC_INITIAL_N } @input) {
     next if $m < $n;
     $delta += ($m - $n) * ($h + 1);
     $n = $m;
 
-    for (my $i = 0; $i < $len; $i++) {
-      my $c = $input[$i];
+    for my $c (@input) {
 
       if ($c < $n) { $delta++ }
       elsif ($c == $n) {
@@ -228,40 +257,27 @@ sub secure_compare {
   return $r == 0;
 }
 
-sub sha1_bytes { sha1 @_ }
-sub sha1_sum   { sha1_hex @_ }
+sub slugify {
+  my ($value, $allow_unicode) = @_;
 
-sub slurp {
-  my $path = shift;
-  croak qq{Can't open file "$path": $!} unless open my $file, '<', $path;
-  my $content = '';
-  while ($file->sysread(my $buffer, 131072, 0)) { $content .= $buffer }
-  return $content;
+  if ($allow_unicode) {
+
+    # Force unicode semantics by upgrading string
+    utf8::upgrade($value = Unicode::Normalize::NFKC($value));
+    $value =~ s/[^\w\s-]+//g;
+  }
+  else {
+    $value = Unicode::Normalize::NFKD($value);
+    $value =~ s/[^a-zA-Z0-9_\p{PosixSpace}-]+//g;
+  }
+  (my $new = lc trim($value)) =~ s/[-\s]+/-/g;
+
+  return $new;
 }
 
 sub split_cookie_header { _header(shift, 1) }
 sub split_header        { _header(shift, 0) }
 
-sub spurt {
-  my ($content, $path) = @_;
-  croak qq{Can't open file "$path": $!} unless open my $file, '>', $path;
-  croak qq{Can't write to file "$path": $!}
-    unless defined $file->syswrite($content);
-  return $content;
-}
-
-sub squish {
-  my $str = trim(@_);
-  $str =~ s/\s+/ /g;
-  return $str;
-}
-
-sub steady_time () {
-  MONOTONIC
-    ? Time::HiRes::clock_gettime(Time::HiRes::CLOCK_MONOTONIC())
-    : Time::HiRes::time;
-}
-
 sub tablify {
   my $rows = shift;
 
@@ -274,8 +290,8 @@ sub tablify {
     }
   }
 
-  my $format = join '  ', map({"\%-${_}s"} @spec[0 .. $#spec - 1]), '%s';
-  return join '', map { sprintf "$format\n", @$_ } @$rows;
+  my @fm = (map({"\%-${_}s"} @spec[0 .. $#spec - 1]), '%s');
+  return join '', map { sprintf join('  ', @fm[0 .. $#$_]) . "\n", @$_ } @$rows;
 }
 
 sub term_escape {
@@ -308,8 +324,18 @@ sub unquote {
 
 sub url_escape {
   my ($str, $pattern) = @_;
-  if   ($pattern) { $str =~ s/([$pattern])/sprintf '%%%02X', ord $1/ge }
-  else            { $str =~ s/([^A-Za-z0-9\-._~])/sprintf '%%%02X', ord $1/ge }
+
+  if ($pattern) {
+    unless (exists $PATTERN{$pattern}) {
+      (my $quoted = $pattern) =~ s!([/\$\[])!\\$1!g;
+      $PATTERN{$pattern}
+        = eval "sub { \$_[0] =~ s/([$quoted])/sprintf '%%%02X', ord \$1/ge }"
+        or croak $@;
+    }
+    $PATTERN{$pattern}->($str);
+  }
+  else { $str =~ s/([^A-Za-z0-9\-._~])/sprintf '%%%02X', ord $1/ge }
+
   return $str;
 }
 
@@ -320,7 +346,8 @@ sub url_unescape {
 }
 
 sub xml_escape {
-  my $str = shift;
+  return $_[0] if ref $_[0] && ref $_[0] eq 'Mojo::ByteStream';
+  my $str = shift // '';
   $str =~ s/([&<>"'])/$XML{$1}/ge;
   return $str;
 }
@@ -336,11 +363,6 @@ sub xor_encode {
   return $output .= $buffer ^ substr($key, 0, length $buffer, '');
 }
 
-sub xss_escape {
-  no warnings 'uninitialized';
-  ref $_[0] eq 'Mojo::ByteStream' ? $_[0] : xml_escape("$_[0]");
-}
-
 sub _adapt {
   my ($delta, $numpoints, $firsttime) = @_;
   use integer;
@@ -356,23 +378,25 @@ sub _adapt {
   return $k + (((PC_BASE - PC_TMIN + 1) * $delta) / ($delta + PC_SKEW));
 }
 
-sub _decode {
-  my ($point, $name) = @_;
+sub _encoding {
+  $ENCODING{$_[0]} //= find_encoding($_[0]) // croak "Unknown encoding '$_[0]'";
+}
+
+sub _entity {
+  my ($point, $name, $attr) = @_;
 
   # Code point
   return chr($point !~ /^x/ ? $point : hex $point) unless defined $name;
 
-  # Find entity name
-  my $rest = '';
+  # Named character reference
+  my $rest = my $last = '';
   while (length $name) {
-    return "$ENTITIES{$name}$rest" if exists $ENTITIES{$name};
-    $rest = chop($name) . $rest;
+    return $ENTITIES{$name} . reverse $rest
+      if exists $ENTITIES{$name}
+      && (!$attr || $name =~ /;$/ || $last !~ /[A-Za-z0-9=]/);
+    $rest .= $last = chop $name;
   }
-  return "&$rest";
-}
-
-sub _encoding {
-  $CACHE{$_[0]} //= find_encoding($_[0]) // croak "Unknown encoding '$_[0]'";
+  return '&' . reverse $rest;
 }
 
 # Supported on Perl 5.14+
@@ -407,6 +431,12 @@ sub _header {
   return [@part ? (@tree, \@part) : @tree];
 }
 
+sub _html {
+  my ($str, $attr) = @_;
+  $str =~ s/$ENTITY_RE/_entity($1, $2, $attr)/geo;
+  return $str;
+}
+
 sub _options {
 
   # Hash or name (one)
@@ -426,15 +456,14 @@ sub _stash {
   my ($name, $object) = (shift, shift);
 
   # Hash
-  my $dict = $object->{$name} ||= {};
-  return $dict unless @_;
+  return $object->{$name} ||= {} unless @_;
 
   # Get
-  return $dict->{$_[0]} unless @_ > 1 || ref $_[0];
+  return $object->{$name}{$_[0]} unless @_ > 1 || ref $_[0];
 
   # Set
   my $values = ref $_[0] ? $_[0] : {@_};
-  @$dict{keys %$values} = values %$values;
+  @{$object->{$name}}{keys %$values} = values %$values;
 
   return $object;
 }
@@ -478,14 +507,14 @@ individually.
 
   my $bytes = b64_decode $b64;
 
-Base64 decode bytes.
+Base64 decode bytes with L<MIME::Base64>.
 
 =head2 b64_encode
 
   my $b64 = b64_encode $bytes;
   my $b64 = b64_encode $bytes, "\n";
 
-Base64 encode bytes, the line ending defaults to a newline.
+Base64 encode bytes with L<MIME::Base64>, the line ending defaults to a newline.
 
 =head2 camelize
 
@@ -524,7 +553,7 @@ Convert a class name to a file.
 
   my $path = class_to_path 'Foo::Bar';
 
-Convert class name to path.
+Convert class name to path, as used by C<%INC>.
 
   # "Foo/Bar.pm"
   class_to_path 'Foo::Bar';
@@ -551,7 +580,8 @@ Convert C<CamelCase> string to C<snake_case> and replace C<::> with C<->.
 
   my $chars = decode 'UTF-8', $bytes;
 
-Decode bytes to characters and return C<undef> if decoding failed.
+Decode bytes to characters with L<Encode>, or return C<undef> if decoding
+failed.
 
 =head2 deprecated
 
@@ -570,17 +600,70 @@ Dump a Perl data structure with L<Data::Dumper>.
 
   my $bytes = encode 'UTF-8', $chars;
 
-Encode characters to bytes.
+Encode characters to bytes with L<Encode>.
+
+=head2 extract_usage
+
+  my $usage = extract_usage;
+  my $usage = extract_usage '/home/sri/foo.pod';
+
+Extract usage message from the SYNOPSIS section of a file containing POD
+documentation, defaults to using the file this function was called from.
+
+  # "Usage: APPLICATION test [OPTIONS]\n"
+  extract_usage;
+
+  =head1 SYNOPSIS
+
+    Usage: APPLICATION test [OPTIONS]
+
+  =cut
+
+=head2 getopt
+
+  getopt
+    'H|headers=s' => \my @headers,
+    't|timeout=i' => \my $timeout,
+    'v|verbose'   => \my $verbose;
+  getopt $array,
+    'H|headers=s' => \my @headers,
+    't|timeout=i' => \my $timeout,
+    'v|verbose'   => \my $verbose;
+  getopt $array, ['pass_through'],
+    'H|headers=s' => \my @headers,
+    't|timeout=i' => \my $timeout,
+    'v|verbose'   => \my $verbose;
+
+Extract options from an array reference with L<Getopt::Long>, but without
+changing its global configuration, defaults to using C<@ARGV>. The configuration
+options C<no_auto_abbrev> and C<no_ignore_case> are enabled by default.
+
+  # Extract "charset" option
+  getopt ['--charset', 'UTF-8'], 'charset=s' => \my $charset;
+  say $charset;
 
 =head2 hmac_sha1_sum
 
   my $checksum = hmac_sha1_sum $bytes, 'passw0rd';
 
-Generate HMAC-SHA1 checksum for bytes.
+Generate HMAC-SHA1 checksum for bytes with L<Digest::SHA>.
 
   # "11cedfd5ec11adc0ec234466d8a0f2a83736aa68"
   hmac_sha1_sum 'foo', 'passw0rd';
 
+=head2 html_attr_unescape
+
+  my $str = html_attr_unescape $escaped;
+
+Same as L</"html_unescape">, but handles special rules from the
+L<HTML Living Standard|https://html.spec.whatwg.org> for HTML attributes.
+
+  # "foo=bar&ltest=baz"
+  html_attr_unescape 'foo=bar&ltest=baz';
+
+  # "foo=bar<est=baz"
+  html_attr_unescape 'foo=bar&lt;est=baz';
+
 =head2 html_unescape
 
   my $str = html_unescape $escaped;
@@ -594,13 +677,13 @@ Unescape all HTML entities in string.
 
   my $checksum = md5_bytes $bytes;
 
-Generate binary MD5 checksum for bytes.
+Generate binary MD5 checksum for bytes with L<Digest::MD5>.
 
 =head2 md5_sum
 
   my $checksum = md5_sum $bytes;
 
-Generate MD5 checksum for bytes.
+Generate MD5 checksum for bytes with L<Digest::MD5>.
 
   # "acbd18db4cc2f85cedef654fccc4a4d8"
   md5_sum 'foo';
@@ -653,22 +736,37 @@ Constant time comparison algorithm to prevent timing attacks.
 
   my $checksum = sha1_bytes $bytes;
 
-Generate binary SHA1 checksum for bytes.
+Generate binary SHA1 checksum for bytes with L<Digest::SHA>.
 
 =head2 sha1_sum
 
   my $checksum = sha1_sum $bytes;
 
-Generate SHA1 checksum for bytes.
+Generate SHA1 checksum for bytes with L<Digest::SHA>.
 
   # "0beec7b5ea3f0fdbc95d0dd47f3c5bc275da8a33"
   sha1_sum 'foo';
 
-=head2 slurp
+=head2 slugify
 
-  my $bytes = slurp '/etc/passwd';
+  my $slug = slugify $string;
+  my $slug = slugify $string, $bool;
 
-Read all data at once from file.
+Returns a URL slug generated from the input string. Non-word characters are
+removed, the string is trimmed and lowercased, and whitespace characters are
+replaced by a dash. By default, non-ASCII characters are normalized to ASCII
+word characters or removed, but if a true value is passed as the second
+parameter, all word characters will be allowed in the result according to
+unicode semantics.
+
+  # "joel-is-a-slug"
+  slugify 'Joel is a slug';
+
+  # "this-is-my-resume"
+  slugify 'This is: my - résumé! ☃ ';
+
+  # "this-is-my-résumé"
+  slugify 'This is: my - résumé! ☃ ', 1;
 
 =head2 split_cookie_header
 
@@ -699,22 +797,6 @@ its own array reference, and keys without a value get C<undef> assigned.
   # "six"
   split_header('one; two="three four", five=six')->[1][1];
 
-=head2 spurt
-
-  $bytes = spurt $bytes, '/etc/passwd';
-
-Write all data at once to file.
-
-=head2 squish
-
-  my $squished = squish $str;
-
-Trim whitespace characters from both ends of string and then change all
-consecutive groups of whitespace into one space each.
-
-  # "foo bar"
-  squish '  foo  bar  ';
-
 =head2 steady_time
 
   my $time = steady_time;
@@ -754,7 +836,7 @@ Trim whitespace characters from both ends of string.
 
   my $unindented = unindent $str;
 
-Unindent multiline string.
+Unindent multi-line string.
 
   # "foo\nbar\nbaz\n"
   unindent "  foo\n  bar\n  baz\n";
@@ -791,26 +873,25 @@ L<RFC 3986|http://tools.ietf.org/html/rfc3986>.
 
   my $escaped = xml_escape $str;
 
-Escape unsafe characters C<&>, C<E<lt>>, C<E<gt>>, C<"> and C<'> in string.
+Escape unsafe characters C<&>, C<E<lt>>, C<E<gt>>, C<"> and C<'> in string, but
+do not escape L<Mojo::ByteStream> objects.
 
   # "&lt;div&gt;"
   xml_escape '<div>';
 
+  # "<div>"
+  use Mojo::ByteStream 'b';
+  xml_escape b('<div>');
+
 =head2 xor_encode
 
   my $encoded = xor_encode $str, $key;
 
 XOR encode string with variable length key.
 
-=head2 xss_escape
-
-  my $escaped = xss_escape $str;
-
-Same as L</"xml_escape">, but does not escape L<Mojo::ByteStream> objects.
-
 =head1 SEE ALSO
 
-L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
 
 =cut
 
