@@ -4,8 +4,10 @@ use Mojo::Base 'Mojo';
 # "Fry: Shut up and take my money!"
 use Carp ();
 use Mojo::Exception;
+use Mojo::Home;
 use Mojo::Log;
 use Mojo::Util;
+use Mojo::UserAgent;
 use Mojolicious::Commands;
 use Mojolicious::Controller;
 use Mojolicious::Plugins;
@@ -24,6 +26,7 @@ has commands => sub {
   return $commands;
 };
 has controller_class => 'Mojolicious::Controller';
+has home             => sub { Mojo::Home->new->detect(ref shift) };
 has log              => sub {
   my $self = shift;
 
@@ -37,6 +40,7 @@ has log              => sub {
   # Reduced log output outside of development mode
   return $mode eq 'development' ? $log : $log->level('info');
 };
+has 'max_request_size';
 has mode => sub { $ENV{MOJO_MODE} || $ENV{PLACK_ENV} || 'development' };
 has moniker  => sub { Mojo::Util::decamelize ref shift };
 has plugins  => sub { Mojolicious::Plugins->new };
@@ -51,13 +55,18 @@ has secrets  => sub {
   # Default to moniker
   return [$self->moniker];
 };
-has sessions  => sub { Mojolicious::Sessions->new };
-has static    => sub { Mojolicious::Static->new };
-has types     => sub { Mojolicious::Types->new };
+has sessions => sub { Mojolicious::Sessions->new };
+has static   => sub { Mojolicious::Static->new };
+has types    => sub { Mojolicious::Types->new };
+has ua       => sub {
+  my $ua = Mojo::UserAgent->new;
+  Scalar::Util::weaken $ua->server->app(shift)->{app};
+  return $ua;
+};
 has validator => sub { Mojolicious::Validator->new };
 
 our $CODENAME = 'Doughnut';
-our $VERSION  = '7.21';
+our $VERSION  = '7.59';
 
 sub AUTOLOAD {
   my $self = shift;
@@ -92,8 +101,12 @@ sub build_controller {
 
 sub build_tx {
   my $self = shift;
-  my $tx   = Mojo::Transaction::HTTP->new;
+
+  my $tx  = Mojo::Transaction::HTTP->new;
+  my $max = $self->max_request_size;
+  $tx->req->max_message_size($max) if defined $max;
   $self->plugins->emit_hook(after_build_tx => $tx, $self);
+
   return $tx;
 }
 
@@ -152,8 +165,8 @@ sub new {
   my $self = shift->SUPER::new(@_);
 
   my $home = $self->home;
-  push @{$self->renderer->paths}, $home->child('templates');
-  push @{$self->static->paths},   $home->child('public');
+  push @{$self->renderer->paths}, $home->child('templates')->to_string;
+  push @{$self->static->paths},   $home->child('public')->to_string;
 
   # Default to controller and application namespace
   my $r = $self->routes->namespaces(["@{[ref $self]}::Controller", ref $self]);
@@ -327,9 +340,10 @@ action dispatching is the last hook in the chain, yours will run before it.
 
 This is a very powerful hook and should not be used lightly, it allows you for
 example to pass additional arguments to actions or handle return values
-differently. (Passed a callback leading to the next hook, the current
-controller object, the action callback and a flag indicating if this action is
-an endpoint)
+differently. Note that this hook can trigger more than once for the same
+request if there are nested routes. (Passed a callback leading to the next hook,
+the current controller object, the action callback and a flag indicating if this
+action is an endpoint)
 
 =head2 before_render
 
@@ -399,6 +413,17 @@ Class to be used for the default controller, defaults to
 L<Mojolicious::Controller>. Note that this class needs to have already been
 loaded before the first request arrives.
 
+=head2 home
+
+  my $home = $app->home;
+  $app     = $app->home(Mojo::Home->new);
+
+The home directory of your application, defaults to a L<Mojo::Home> object
+which stringifies to the actual path.
+
+  # Portably generate path relative to home directory
+  my $path = $app->home->child('data', 'important.txt');
+
 =head2 log
 
   my $log = $app->log;
@@ -412,6 +437,18 @@ if a C<log> directory exists.
   # Log debug message
   $app->log->debug('It works');
 
+=head2 max_request_size
+
+  my $max = $app->max_request_size;
+  $app    = $app->max_request_size(16777216);
+
+Maximum request size in bytes, defaults to the value of
+L<Mojo::Message/"max_message_size">. Setting the value to C<0> will allow
+requests of indefinite size. Note that increasing this value can also
+drastically increase memory usage, should you for example attempt to parse an
+excessively large request body with the methods L<Mojo::Message/"dom"> or
+L<Mojo::Message/"json">.
+
 =head2 mode
 
   my $mode = $app->mode;
@@ -521,6 +558,9 @@ L<Mojolicious::Static> object.
   # Add another class with static files in DATA section
   push @{$app->static->classes}, 'Mojolicious::Plugin::Fun';
 
+  # Remove built-in favicon
+  delete $app->static->extra->{'favicon.ico'};
+
 =head2 types
 
   my $types = $app->types;
@@ -532,6 +572,17 @@ L<Mojolicious::Types> object.
   # Add custom MIME type
   $app->types->type(twt => 'text/tweet');
 
+=head2 ua
+
+  my $ua = $app->ua;
+  $app   = $app->ua(Mojo::UserAgent->new);
+
+A full featured HTTP user agent for use in your applications, defaults to a
+L<Mojo::UserAgent> object.
+
+  # Perform blocking request
+  say $app->ua->get('example.com')->result->body;
+
 =head2 validator
 
   my $validator = $app->validator;
@@ -751,12 +802,6 @@ have been used in the past.
 
 2.0, C<Leaf Fluttering In Wind> (U+1F343)
 
-1.4, C<Smiling Face With Sunglasses> (U+1F60E)
-
-1.3, C<Tropical Drink> (U+1F379)
-
-1.1, C<Smiling Cat Face With Heart-Shaped Eyes> (U+1F63B)
-
 1.0, C<Snowflake> (U+2744)
 
 =head1 SPONSORS
@@ -774,10 +819,6 @@ Current members of the core team in alphabetical order:
 
 =over 2
 
-Abhijit Menon-Sen, C<ams@cpan.org>
-
-Glen Hinkle, C<tempire@cpan.org>
-
 Jan Henning Thorsen, C<jhthorsen@cpan.org>
 
 Joel Berger, C<jberger@cpan.org>
@@ -786,6 +827,16 @@ Marcus Ramberg, C<mramberg@cpan.org>
 
 =back
 
+The following members of the core team are currently on hiatus:
+
+=over 2
+
+Abhijit Menon-Sen, C<ams@cpan.org>
+
+Glen Hinkle, C<tempire@cpan.org>
+
+=back
+
 =head1 CREDITS
 
 In alphabetical order:
@@ -854,6 +905,8 @@ Charlie Brady
 
 Chas. J. Owens IV
 
+Chase Whitener
+
 Christian Hansen
 
 chromatic
@@ -882,8 +935,12 @@ Dominik Jarmulowicz
 
 Dominique Dumont
 
+Dotan Dimet
+
 Douglas Christopher Wilson
 
+Ettore Di Giacinto
+
 Eugen Konkov
 
 Eugene Toropov
@@ -1016,6 +1073,10 @@ Roy Storey
 
 Ryan Jendoubi
 
+Salvador Fandino
+
+Santiago Zarate
+
 Sascha Kiefer
 
 Scott Wiersdorf
@@ -1034,8 +1095,12 @@ Stanis Trendelenburg
 
 Steffen Ullrich
 
+Stephan Kulow
+
 Stephane Este-Gracias
 
+Stevan Little
+
 Steve Atkins
 
 Tatsuhiko Miyagawa
@@ -1046,6 +1111,8 @@ Tianon Gravi
 
 Tomas Znamenacek
 
+Tudor Constantin
+
 Ulrich Habel
 
 Ulrich Kautz
@@ -1060,6 +1127,8 @@ Viliam Pucik
 
 Wes Cravens
 
+William Lindley
+
 Yaroslav Korshak
 
 Yuki Kimoto
