@@ -21,8 +21,7 @@ sub import {
   push @{"${caller}::ISA"}, 'Mojo';
 
   # Generate moniker based on filename
-  my $moniker = path($ENV{MOJO_EXE})->basename;
-  $moniker =~ s/\.(?:pl|pm|t)$//i;
+  my $moniker = path($ENV{MOJO_EXE})->basename('.pl', '.pm', '.t');
   my $app = shift->new(moniker => $moniker);
 
   # Initialize routes without namespaces
@@ -34,8 +33,7 @@ sub import {
   for my $name (qw(any get options patch post put websocket)) {
     monkey_patch $caller, $name, sub { $routes->$name(@_) };
   }
-  monkey_patch $caller, $_, sub {$app}
-    for qw(new app);
+  monkey_patch($caller, $_, sub {$app}) for qw(new app);
   monkey_patch $caller, del => sub { $routes->delete(@_) };
   monkey_patch $caller, group => sub (&) {
     (my $old, $root) = ($root, $routes);
@@ -52,7 +50,8 @@ sub import {
   Mojo::UserAgent::Server->app($app) unless Mojo::UserAgent::Server->app;
 
   # Lite apps are strict!
-  Mojo::Base->import(-strict);
+  unshift @_, 'Mojo::Base', -strict;
+  goto &Mojo::Base::import;
 }
 
 1;
@@ -80,11 +79,117 @@ Mojolicious::Lite - Micro real-time web framework
 
 =head1 DESCRIPTION
 
-L<Mojolicious::Lite> is a micro real-time web framework built around
-L<Mojolicious>.
+L<Mojolicious::Lite> is a tiny domain specific language built around
+L<Mojolicious>, made up of only about a dozen Perl functions.
+
+On Perl 5.20+ you can also use a C<-signatures> flag to enable support for
+L<subroutine signatures|perlsub/"Signatures">.
+
+  use Mojolicious::Lite -signatures;
+
+  get '/:foo' => sub ($c) {
+    my $foo = $c->param('foo');
+    $c->render(text => "Hello from $foo.");
+  };
+
+  app->start;
 
 See L<Mojolicious::Guides::Tutorial> for more!
 
+=head1 GROWING
+
+While L<Mojolicious::Guides::Growing> will give you a detailed introduction to
+growing a L<Mojolicious::Lite> prototype into a well-structured L<Mojolicious>
+application, here we have collected a few snippets that illustrate very well
+just how similar both of them are.
+
+=head2 Routes
+
+The functions L</"get">, L</"post"> and friends all have equivalent methods.
+
+  # Mojolicious::Lite
+  get '/foo' => sub {
+    my $c = shift;
+    $c->render(text => 'Hello World!');
+  };
+
+  # Mojolicious
+  sub startup {
+    my $self = shift;
+
+    my $routes = $self->routes;
+    $routes->get('/foo' => sub {
+      my $c = shift;
+      $c->render(text => 'Hello World!');
+    });
+  }
+
+=head2 Application
+
+The application object you can access with the function L</"app"> is the first
+argument passed to the C<startup> method.
+
+  # Mojolicious::Lite
+  app->max_request_size(16777216);
+
+  # Mojolicious
+  sub startup {
+    my $self = shift;
+    $self->max_request_size(16777216);
+  }
+
+=head2 Plugins
+
+Instead of the L</"plugin"> function you just use the method
+L<Mojolicious/"plugin">.
+
+  # Mojolicious::Lite
+  plugin 'Config';
+
+  # Mojolicious
+  sub startup {
+    my $self = shift;
+    $self->plugin('Config');
+  }
+
+=head2 Helpers
+
+Similar to plugins, instead of the L</"helper"> function you just use the method
+L<Mojolicious/"helper">.
+
+  # Mojolicious::Lite
+  helper two => sub {
+    my $c = shift;
+    return 1 + 1;
+  };
+
+  # Mojolicious
+  sub startup {
+    my $self = shift;
+    $self->helper(two => sub {
+      my $c = shift;
+      return 1 + 1;
+    });
+  }
+
+=head2 Under
+
+Instead of sequential function calls, we can use methods to build a tree with
+nested routes, that much better illustrates how routes work internally.
+
+  # Mojolicious::Lite
+  under '/foo';
+  get '/bar' => sub {...};
+
+  # Mojolicious
+  sub startup {
+    my $self = shift;
+
+    my $routes = $self->routes;
+    my $foo = $routes->under('/foo');
+    $foo->get('/bar' => sub {...});
+  }
+
 =head1 FUNCTIONS
 
 L<Mojolicious::Lite> implements the following functions, which are
